#!/usr/bin/env python
"""
 run an model using a socket interface

 does this in two steps : 
  1. generates jar file from a model file which implements that model
  2. runs that jar file

"""
# Author: Harco Kuppens


import sys, re, pprint,os  # modules from standard lib (python 2.6 and later)

import tempfile
import shutil
import argparse

import util
import uppaal2jar




def check_dependencies():
    util.check_program('java','.exe')
    util.check_program('javac','.exe')

def args_parsing(args):

    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    argparser.add_argument('modelfile', help='input uppaal model file')
    argparser.add_argument('-v', '--verbose',action='store_true',help='turn verbose mode on : displays all input and outputs')
    argparser.add_argument( '--port',action='store',help='tcp port  to listen on for incoming connections')

    parse_result=argparser.parse_args(args[1:])

    verbose=parse_result.verbose
    port=parse_result.port

    # handle input/output file argument
    modelfile=os.path.abspath(parse_result.modelfile).replace('\\','/')
    util.check_isreadablefile(modelfile)

    return  (modelfile,verbose,port)


def run(modelfile,verbose,port):
    """
	 run an model using a socket interface

	 does this in two steps : 
	  1. generates jar file from a model file which implements that model
	  2. runs that jar file
    """

    # 1. generate sut.jar from model
    #------------------------------------

    # create tmp directory to generate sut.jar in
    tmpdir=tempfile.mkdtemp(prefix="sut_run_")

    # create sut.jar file
    jarfile=tmpdir + '/sut.jar'
    uppaal2jar.uppaal2jar(modelfile,jarfile)


    # 2. run sut.jar 
    #----------------------------
    command=[ 'java', '-jar', jarfile]
    # pass through optionally commands
    if port: command+=['--port',str(port)]
    if verbose: command.append('--verbose')

    util.run_command(command, cwd='/')

    # 3. cleanup
    #----------------------------
    #print "results in directory: " +  tmpdir
    shutil.rmtree(tmpdir)


def main(args):
    """
	 run an model using a socket interface

	 does this in two steps : 
	  1. generates jar file from a model file which implements that model
	  2. runs that jar file
    """
    check_dependencies()

    (modelfile,verbose,port)=args_parsing(args)
    run(modelfile,verbose,port)



if __name__ == "__main__":
    print main(sys.argv)

