#!/bin/env python
"""Generate sut java implementation from uppaal model"""
# Author: Harco Kuppens


import sys, os,  pprint, re, getpass, tempfile, subprocess, shutil  # standard libraries
import argparse # only in standard lib for python 2.7 and later -> installed local copy  (pure python versions)
import jinja2, yaml  # external libraries   -> installed local copy   (pure python versions) (not yaml doesn't use libyaml but pure python version)
import pyuppaal  # local libraries          -> installed local copy   (pure python versions)



import model_parser
import get_models

import gentemplate


import util
import uppaal2lotos










#-------------------------------------------------------------------------------
#   steps
#-------------------------------------------------------------------------------




def lotos2bcg(lotos_dir,bcg_file):

    #command="svl model.svl"  # doesn't work: [Errno 2] No such file or directory -> because we don't use shell-> arg is not interpreted but seen as a single command
    command=["svl","model.svl"]

    output_str=util.run_command_with_output(command,cwd=lotos_dir)
    shutil.copy(os.path.join(lotos_dir, 'model.bcg'), bcg_file)

    return output_str 


def uppaal2bcg(model_file,bcg_file,range_min,range_max,keeptmp=False):

    import tempfile
    tmpdir=tempfile.mkdtemp(prefix="tomte_uppaal2bcg_")
    shutil.rmtree(tmpdir)
    uppaal2lotos.uppaal2lotos(model_file,tmpdir,range_min,range_max)
    output_str=lotos2bcg(tmpdir,bcg_file)
    if not keeptmp: 
       shutil.rmtree(tmpdir)

    return output_str 


#-------------------------------------------------------------------------------
#   main + input parameters parsing
#-------------------------------------------------------------------------------


def args_parsing(args):

    #parser = argparse.ArgumentParser(prog=os.path.basename(sys.argv[0]),epilog=get_models_help(models))  # prints epilog always on single line
    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    argparser.add_argument('modelfile', help='input uppaal model file')
    argparser.add_argument('bcgfile', help='bcg output file')
    argparser.add_argument('range_min', help='minimum value of range used for input params when flattening the model')
    argparser.add_argument('range_max', help='maximum value of range used for input params when flattening the model')

    str=argparser.format_help()

    def myhelp( ):
        print str  + "\n" +  get_models.get_models_help(models)

    def myerror( message):
            print >>sys.stderr, ('%s: error: %s\n') % (argparser.prog, message)
            argparser.print_usage(sys.stderr)
            exit(1)

    argparser.error=myerror
    argparser.format_help=myhelp

    parse_result=argparser.parse_args(args[1:])

    # handle modelfile argument
    model_file=parse_result.modelfile
    # always use unix slashes in python!!
    # automatic convert any windows slashes
    # TODO: document this convention in python/txtdocs/
    model_file.replace('\\','/')
    util.check_isreadablefile(model_file)


    # handle output directory argument
    # - first get absolute path
    bcg_file=parse_result.bcgfile
    bcg_file=os.path.abspath(bcg_file)   # on windows also interprets / as dir sep, and returns path with windows \
    bcg_file.replace('\\','/')



    range_min=parse_result.range_min
    range_max=parse_result.range_max

    return  [model_file,bcg_file,range_min,range_max]




def main(args):
    util.check_program('svl')

    #parse arguments
    (model_file,bcg_file,range_min,range_max)=args_parsing(args)


    # convert uppaal model 2 bcg
    print uppaal2bcg(model_file,bcg_file,range_min,range_max)


if __name__ == "__main__":
    main(sys.argv)


