#!/usr/bin/env python
""" 
  generate java source files from a model file which implement that model
  using a socket interface
"""
# Author: Harco Kuppens


import sys, re, pprint,os  # modules from standard lib (python 2.6 and later)

import shutil
import argparse

import util
import uppaal2sutjavafile



def check_dependencies():
    util.check_program('java','.exe')


def args_parsing(args):
    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    argparser.add_argument('modelfile', help='input uppaal model file')
    argparser.add_argument('srcdir', help='directory where java source implementing model is generated')

    parse_result=argparser.parse_args(args[1:])


    # handle input/output file argument
    modelfile=os.path.abspath(parse_result.modelfile).replace('\\','/')
    srcdir=os.path.abspath(parse_result.srcdir).replace('\\','/')
    # Note: on windows also interprets / as dir sep, and returns path with windows \


    util.check_isreadablefile(modelfile)
    util.check_noneexistingdir(srcdir)


    return  [modelfile,srcdir]



def uppaal2java(modelfile,srcdir):
    """ 
      generate java source files from a model file which implement that model
      using a socket interface
    """
    sep=os.path.sep

    # resource dir is relative from this module :
    resource_dir = os.path.abspath( os.path.dirname(os.path.abspath(__file__)) + '/../resources/' )

    # copy static src directory from resource dir which contains fixed code need to build jar file
    static_src= resource_dir  + sep +  'sutjar' +  sep + 'src'
    ignore_svn_folders=shutil.ignore_patterns('.svn')
    shutil.copytree(static_src,srcdir,ignore=ignore_svn_folders)

    # create java sut file
    sutfile= srcdir + sep + 'sut' + sep + 'Sut.java'
    uppaal2sutjavafile.uppaal2sutjavafile(modelfile,sutfile)


def main(args):
    """
      generate java source files from a model file which implement that model
      using a socket interface
    """
    check_dependencies()

    (modelfile,srcdir)=args_parsing(args)
    uppaal2java(modelfile,srcdir)


if __name__ == "__main__":
    print main(sys.argv)

